local MinBuild, MaxBuild, Build = 40000, 0, select(4, GetBuildInfo())
if Build < MinBuild or ( MaxBuild > 0 and Build > MaxBuild ) then return end
local AddonName, a = ...
local AddonTitle = select(2, GetAddOnInfo(AddonName))
local PlainAddonTitle = AddonTitle:gsub("|c........", ""):gsub("|r", "")
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables

local c = BittensSpellFlashLibrary
a.Rotations = {}

s.Spam[AddonName] = function()
    c.Flash(a)
end

local rp = 0
local rpBump = 0
local rpBumpTime = 0
function a.BumpRP(amount)
    rpBump = s.Power("player") + amount
    rpBumpTime = GetTime() + .6
end

local function updateRP()
    rp = s.Power("player")
    if rp < rpBump and GetTime() < rpBumpTime then
        rp = rpBump
    end
end

local spellIfCapped
local function flashNoCap(wouldAdd, name)
    local spell = a.spells[name]
    if s.Flashable(spell.ID) and s.Castable(spell) then
        if s.MaxPower("player") - rp > wouldAdd then
            s.Flash(spell.ID)
            return not spell.Continue
        elseif spellIfCapped ~= nil then
            s.Flash(c.GetID(spellIfCapped))
            return true
        end
    end
    return false
end

------------------------------------------------------------------------- Blood
local uncontrolledMitigationBuffs = {
    109774, -- Master Tactician LFR
    107986, -- Master Tactician
    109776, -- Master Tactician Heroic
}

a.Rotations.Blood = {
    Spec = 1,
    OffSwitch = "blood_off",
    Function = function()
        a.Flash("Blood Presence")    
        if not s.InCombat() then
            a.Flash("Bone Shield")
            return
        end
        
        -- non-GCD cooldowns
        a.FlashAll("Rune Tap", "Mind Freeze", "Dark Command", "Death Grip")
        c.RotateCooldowns(
            uncontrolledMitigationBuffs,
            "Dancing Rune Weapon",
            "Bone Shield",
            "Vampiric Blood",
            "Fire of the Deep",
            "Icebound Fortitude")
        
        -- rotation
        a.Flash(
            "Death Strike for Health",
            "Death Strike to Save Shield",
            "Blood Tap for Death Strike for Health",
            "Outbreak for Debuffs",
            "Icy Touch for Debuff",
            "Plague Strike for Debuff",
            "Death Strike if Two Available",
            "Rune Strike if Capped",
            "Rune Strike for Runic Empowerment",
            "Heart Strike",
            "Free Blood Boil",
            "HoW",
            "Outbreak")
    end
}

------------------------------------------------------------------------- Frost
local obliterateDelay = GetTime()
function a.DelayObliterate()
    obliterateDelay = GetTime() + .9
end
local kmDelay = GetTime()
function a.DelayKM()
    kmDelay = GetTime() + .6
end
a.Rotations.Frost = {
    Spec = 2,
    OffSwitch = "frost_off",
    Function = function()
        a.Flash("Unholy Presence")
        if not s.InCombat() then
            return
        end
        
        updateRP()
        spellIfCapped = "FS"
        
        -- non-GCD cooldowns
        a.FlashAll(
            "Pillar of Frost", 
            "Blood Tap", 
            "Empower Rune Weapon when low",
            "Mind Freeze")
        
        -- diseases
        local km = GetTime() > kmDelay and c.HasBuff("Killing Machine")
        if km and a.Flash("FS if missing disease") then
            return
        end
        if a.Flash("Outbreak") then
            return
        end
        if flashNoCap(10, "Rime for FF") then
            return
        end
        if flashNoCap(20, "HB for FF") then
            return
        end
        if flashNoCap(10, "PS for BP unless Outbreak") then
            return
        end
        
        -- killing machine
        if km then
            if flashNoCap(30, "OB no Rime") then
                return
            end
            if a.Flash("FS") then
                return
            end
        end
        
        -- obliterate
        local toFill = s.MaxPower("player") - rp
        if a.BothRunesAvailable("Unholy", 1) then
            if toFill <= 30 then
                a.Flash("FS")
                return
            elseif a.Flash("OB") then
                return
            end
        end
        if a.BothRunesAvailable("Unholy", 2) then
            if c.HasBuff("Freezing Fog") then
                if toFill <= 10 then
                    a.Flash("FS")
                    return
                end
            elseif toFill <= 20 then
                a.Flash("FS")
                return
            end
        end
        
        -- howling blast
        if flashNoCap(10, "Rime") then
            return
        end
        if flashNoCap(20, "HB") then
            return
        end
        
        -- frost strike
        a.Flash("FS", "HoW")
    end
}

------------------------------------------------------------------------ Unholy
a.Rotations.Unholy = {
    Spec = 3,
    OffSwitch = "unholy_off",
    Function = function()
        a.FlashAll("Unholy Presence", "Raise Dead")
        if not s.InCombat() then
            return
        end
        
        if s.BuffStack(c.GetID("Shadow Infusion"), "pet") == 4
            and c.IsAuraPendingFor("DC") then
            
            return
        end
        
        updateRP()
        local dtOff = s.BuffDuration(c.GetID("Dark Transformation"), "pet")
            - s.GlobalCooldown()
        local unrestricted = dtOff <= 0 or dtOff > 2
        if unrestricted then
            spellIfCapped = "DC"
        else
            spellIfCapped = nil
        end
        
        -- non-GCD cooldowns
        a.FlashAll(
            "Blood Tap",
            "Empower Rune Weapon when low",
            "Mind Freeze",
            "Summon Gargoyle")
        
        -- flashing
        if a.Flash("Outbreak", "Dark Transformation") then
            return
        end
        if rp >= s.MaxPower("player") - 10 then
            if unrestricted then
                a.Flash("DC")
            end
            return
        end
        if a.Flash(
            "IT for FF",
            "PS for BP",
            "DC for Shadow Infusion",
            "DnD DB or DF or UU",
            "SS DB or DF or UU") then
            
            return
        end
        if flashNoCap(20, "FeS BBFF") then
            return
        end
        if unrestricted and a.Flash("DC Sudden Doom") then
            return
        end
        if a.Flash("DnD", "SS") then
            return
        end
        if flashNoCap(20, "FeS") then
            return
        end
        a.Flash("HoW", "Unholy Frenzy")
    end
}
