local MinBuild, MaxBuild, Build = 40000, 0, select(4, GetBuildInfo())
if Build < MinBuild or ( MaxBuild > 0 and Build > MaxBuild ) then return end
local AddonName, a = ...
local AddonTitle = select(2, GetAddOnInfo(AddonName))
local PlainAddonTitle = AddonTitle:gsub("|c........", ""):gsub("|r", "")
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables

local c = BittensSpellFlashLibrary

------------------------------------------------------------------------ common
a.spells["Frost Fever"] = { ID = 59921 }
a.spells["Blood Plague"] = { ID = 59879 }

local function getRuneCd(runeId)
    local start, duration = GetRuneCooldown(runeId)
    return math.max(0, start + duration - s.GlobalCooldown() - GetTime())
end

local function isFullyDepleted(runeId)
    local start, duration = GetRuneCooldown(runeId)
    return start > GetTime() + s.GlobalCooldown()
end

local function runeAvailable(runeId, delay)
    if delay == nil then
        delay = 0
    end
    return getRuneCd(runeId) <= delay
end

local function isDeathRune(runeId)
    return GetRuneType(runeId) == 4
end

function a.BothRunesAvailable(runeName, delay)
    if runeName == "Blood" then
        return runeAvailable(1, delay) and runeAvailable(2, delay)
    end
    if runeName == "Unholy" then
        return runeAvailable(3, delay) and runeAvailable(4, delay)
    end
    if runeName == "Frost" then
        return runeAvailable(5, delay) and runeAvailable(6, delay)
    end
    print("Bad rune name: ", runeName)
end

local function hasFrostFever()
    return c.HasMyDebuff("Frost Fever")
        or c.IsAuraPendingFor("Outbreak")
        or c.IsAuraPendingFor("IT")
        or (s.HasGlyph(63335--[[Glyph of Howling Blast]])
            and c.IsAuraPendingFor("HB"))
end

local function hasBloodPlague()
    return c.HasMyDebuff("Blood Plague")
        or c.IsAuraPendingFor("Outbreak")
        or c.IsAuraPendingFor("PS")
end

local function hasBothDiseases()
    return hasFrostFever() and hasBloodPlague()
end

a.spells["HoW"] = {
    ID = 57330,
    CastTime = 0,
    CheckFirst = function()
        return s.MaxPower("player") - s.Power("player") >= 10
    end
}

a.spells["Empower Rune Weapon"] = {
    ID = 47568,
    NoGCD = true,
    FlashColor = "yellow",
}

a.spells["Empower Rune Weapon when low"] = {
    ID = 47568,
    NoGCD = true,
    FlashColor = "yellow",
    CheckFirst = function()
        return not runeAvailable(1)
            and not runeAvailable(2)
            and not runeAvailable(3)
            and not runeAvailable(4)
            and not runeAvailable(5)
            and not runeAvailable(6)
            and s.Power("player") < 32
    end
}

a.spells["Outbreak"] = {
    ID = 77575,
    CheckFirst = function()
        return not hasBothDiseases()
    end
}

a.spells["Blood Tap"] = {
    ID = 45529,
    NoGCD = true,
    FlashColor = "yellow",
    CheckFirst = function()
        return isFullyDepleted(1)
    end
}

a.spells["PS"] = {
    ID = 45462,
}

a.spells["Mind Freeze"] = {
    ID = 47528,
    Interrupt = 1,
    FlashColor = "aqua",
}

------------------------------------------------------------------------- blood
a.spells["Blood Shield"] = { ID = 77535 }
a.spells["Crimson Scourge"] = { ID = 81141 }

local function shouldRuneTap()
    return s.HealthPercent("player") <= 90
end

local function shouldDeathStrikeForHealth()
    return s.HealthPercent("player") <= 93
end

local function canBloodTapForDeathStrike()
    local frost = 0
    local unholy = 0
    local death = 0
    if runeAvailable(3) then
        if isDeathRune(3) then
            death = death + 1
        else
            unholy = unholy + 1
        end
    end
    if runeAvailable(4) then
        if isDeathRune(4) then
            death = death + 1
        else
            unholy = unholy + 1
        end
    end
    if runeAvailable(5) then
        if isDeathRune(5) then
            death = death + 1
        else
            frost = frost + 1
        end
    end
    if runeAvailable(6) then
        if isDeathRune(6) then
            death = death + 1
        else
            frost = frost + 1
        end
    end
    return (frost + unholy + death == 1)
        or (death == 0 and (
                (frost == 0 and unholy > 0)
            or (unholy == 0 and frost > 0)))
end

a.spells["Blood Presence"] = {
    Type = "form",
    ID = 48263,
    NotIfActive = 1,
    FlashColor = "yellow",
}

a.spells["Outbreak for Debuffs"] = {
    ID = 77575,
    MyDebuff1 = c.SLOW_MELEE_DEBUFFS,
    MyDebuff2 = c.PHYSICAL_DAMAGE_DEBUFFS,
}

a.spells["Icy Touch for Debuff"] = {
    ID = 45477,
    Debuff = c.SLOW_MELEE_DEBUFFS,
    CheckFirst = function()
        return c.GetCooldown("Outbreak") > 3
    end
}

a.spells["Plague Strike for Debuff"] = {
    ID = 45462,
    Debuff = c.PHYSICAL_DAMAGE_DEBUFFS,
    CheckFirst = function()
        return c.GetCooldown("Outbreak") > 3
    end
}

a.spells["Heart Strike"] = {
    ID = 55050,
    CheckFirst = function()
        if runeAvailable(1) then
            return not isDeathRune(1) and runeAvailable(2, 1.5)
        end
        return runeAvailable(2) and not isDeathRune(2) and runeAvailable(1, 1.5)
    end
}

a.spells["Death Strike"] = {
    ID = 49998,
}

a.spells["Death Strike for Health"] = {
    ID = 49998,
    CheckFirst = function()
        return shouldDeathStrikeForHealth()
    end
}

a.spells["Death Strike for Runic Empowerment"] = {
    ID = 49998,
    CheckFirst = function()
        return not ( -- keep at least one rune fully depleted
            isFullyDepleted(3) 
            or isFullyDepleted(4) 
            or isFullyDepleted(5) 
            or isFullyDepleted(6))
    end
}

a.spells["Death Strike for Rune Cooldown"] = {
    ID = 49998,
    CheckFirst = function()
        return a.BothRunesAvailable("Frost") or a.BothRunesAvailable("Unholy")
    end
}

a.spells["Death Strike to Save Shield"] = {
    ID = 49998,
    CheckFirst = function()
        return c.GetBuffDuration("Blood Shield") < 2
    end
}

a.spells["Death Strike if Two Available"] = {
    ID = 49998,
    CheckFirst = function()
        local avail = 0
        if runeAvailable(3, 1.5) then
            avail = avail + 1
        end
        if runeAvailable(4, 1.5) then
            avail = avail + 1
        end
        if runeAvailable(5, 1.5) then
            avail = avail + 1
        end
        if runeAvailable(6, 1.5) then
            avail = avail + 1
        end
        return avail >= 4
    end
}

a.spells["Rune Strike for Runic Empowerment"] = {
    ID = 56815,
    CheckFirst = function()
        return isFullyDepleted(3)
            or isFullyDepleted(4)
            or isFullyDepleted(5)
            or isFullyDepleted(6)
    end
}

a.spells["Rune Strike if Capped"] = {
    ID = 56815,
    CheckFirst = function()
        return s.MaxPower("player") - s.Power("player") < 20
    end
}

a.spells["Rune Tap"] = {
    ID = 48982,
    NoGCD = true,
    FlashColor = "yellow",
    CheckFirst = function()
        return shouldRuneTap()
    end
}

a.spells["Bone Shield"] = {
    ID = 49222,
    FlashColor = "yellow",
    CheckFirst = function()
        if s.InCombat() then
            return not c.HasBuff("Bone Shield")
        else
            return c.GetBuffDuration("Bone Shield") < 2 * 60
                or s.BuffStack(c.GetID("Bone Shield"), "player") < 6
        end
    end
}

a.spells["Dancing Rune Weapon"] = {
    ID = 49028,
    FlashColor = "yellow",
}

a.spells["Vampiric Blood"] = {
    ID = 55233,
    NoGCD = true,
    FlashColor = "yellow",
    CheckFirst = function()
        return s.HealthPercent("player") < 90
    end
}

a.spells["Icebound Fortitude"] = {
    ID = 48792,
    NoGCD = true,
    FlashColor = "yellow",
}

a.spells["Blood Tap for Death Strike for Health"] = {
    ID = 45529,
    NoGCD = true,
    CheckFirst = function()
        return shouldDeathStrikeForHealth() and canBloodTapForDeathStrike()
    end
}

a.spells["Free Blood Boil"] = {
    ID = 48721,
    CheckFirst = function()
        return s.MaxPower("player") - s.Power("player") >= 10
            and c.HasBuff("Crimson Scourge")
    end
}

a.spells["Dark Command"] = {
    ID = 56222,
    NoGCD = true,
    CheckFirst = c.CheckFirstForTaunts,
}

a.spells["Death Grip"] = {
    ID = 49576,
    NoGCD = true,
    CheckFirst = c.CheckFirstForTaunts,
}

a.spells["Elusive"] = { ID = 107951 }
a.spells["Fire of the Deep"] = {
    Type = "item",
    ID = 77117,
    Buff = "Elusive",
    FlashColor = "yellow",
}

------------------------------------------------------------------------- frost
a.spells["Freezing Fog"] = { ID = 59052 }
a.spells["Killing Machine"] = { ID = 51124 }

a.spells["Unholy Presence"] = {
    Type = "form",
    ID = 48265,
    NotIfActive = 1,
    FlashColor = "yellow",
}

a.spells["FS"] = {
    ID = 49143,
}

a.spells["FS if missing disease"] = {
    ID = 49143,
    CheckFirst = function()
        return not hasBothDiseases()
    end
}

a.spells["HB"] = {
    ID = 49184,
}

a.spells["Rime"] = {
    ID = 49184,
    CheckFirst = function()
        return c.HasBuff("Freezing Fog")
    end
}

a.spells["Rime for FF"] = {
    ID = 49184,
    CheckFirst = function()
        return c.HasBuff("Freezing Fog") and not hasFrostFever()
    end
}

a.spells["HB for FF"] = {
    ID = 49184,
    CheckFirst = function()
        return not hasFrostFever()
    end
}

a.spells["PS for BP unless Outbreak"] = {
    ID = 45462,
    CheckFirst = function()
        return c.GetCooldown("Outbreak") > 3 and not hasBloodPlague()
    end
}

a.spells["OB"] = {
    ID = 49020,
    CheckFirst = function()
        return hasBothDiseases()
    end
}

a.spells["OB no Rime"] = {
    ID = 49020,
    CheckFirst = function()
        return not c.HasBuff("Freezing Fog") and hasBothDiseases()
    end
}

a.spells["OB UU"] = {
    ID = 49020,
    CheckFirst = function()
        return a.BothRunesAvailable("Unholy", 1) and hasBothDiseases()
    end
}

a.spells["OB BBFF"] = {
    ID = 49020,
    CheckFirst = function()
        return a.BothRunesAvailable("Blood", 1) 
            and a.BothRunesAvailable("Frost", 1)
            and hasBothDiseases()
    end
}

a.spells["Pillar of Frost"] = {
    ID = 51271,
    NoGCD = true,
    FlashColor = "yellow",
}

a.spells["Raise Dead"] = {
    ID = 46584,
}

------------------------------------------------------------------------ Unholy
a.spells["IT"] = { ID = 45477 }
a.spells["Shadow Infusion"] = { ID = 91342 }

a.spells["Raise Dead"] = {
    ID = 46584,
    FlashColor = "yellow",
    NoRangeCheck = 1,
    CheckFirst = function()
        return not UnitExists("pet") and not IsMounted()
    end
}

a.spells["Unholy Frenzy"] = {
    ID = 49016,
    NoRangeCheck = 1,
    CheckFirst = function()
        return not s.Buff(c.BLOODLUST_BUFFS, "player")
    end
}

a.spells["Summon Gargoyle"] = {
    ID = 49206,
    FlashColor = "yellow",
    CheckFirst = function()
        return s.BuffDuration(c.GetID("Dark Transformation"), "pet")
            - s.GlobalCooldown() > 10
    end
}

a.spells["IT for FF"] = {
    ID = 45477,
    CheckFirst = function()
        return not hasFrostFever()
    end
}

a.spells["PS for BP"] = {
    ID = 45462,
    CheckFirst = function()
        return not hasBloodPlague()
    end
}

a.spells["Dark Transformation"] = {
    ID = 63560,
    Buff = 63560,
    BuffUnit = "pet",
}

a.spells["DC"] = {
    ID = 47541,
}

a.spells["DC for Shadow Infusion"] = {
    ID = 47541,
    CheckFirst = function()
        return not s.Buff(c.GetID("Dark Transformation"), "pet", c.GetBusyTime())
            and s.BuffStack(c.GetID("Shadow Infusion"), "pet") < 5
    end
}

a.spells["DC Sudden Doom"] = {
    ID = 47541,
    RequireBuff = 81340,
    RequireBuffUnit = "player",
    ConsumeBuff = 81340,
    ConsumeBuffUnit = "player",
}

a.spells["DnD"] = {
    ID = 43265,
    NoRangeCheck = 1,
    FlashColor = "yellow",
    Continue = 1,
}

local function DB_or_DF_or_UU()
    return a.BothRunesAvailable("Unholy", 1)
        or (isDeathRune(1) and runeAvailable(1, 0) and runeAvailable(2, 1))
        or (isDeathRune(2) and runeAvailable(2, 0) and runeAvailable(1, 1))
        or (isDeathRune(5) and runeAvailable(5, 0) and runeAvailable(6, 1))
        or (isDeathRune(6) and runeAvailable(6, 0) and runeAvailable(5, 1))
end

a.spells["DnD DB or DF or UU"] = {
    ID = 43265,
    NoRangeCheck = 1,
    FlashColor = "yellow",
    Continue = 1,
    CheckFirst = DB_or_DF_or_UU,
}

a.spells["SS"] = {
    ID = 55090,
}

a.spells["SS DB or DF or UU"] = {
    ID = 55090,
    CheckFirst = DB_or_DF_or_UU,
}

a.spells["FeS"] = {
    ID = 85948,
}

a.spells["FeS BBFF"] = {
    ID = 85948,
    CheckFirst = function()
        return a.BothRunesAvailable("Blood", 1)
            or a.BothRunesAvailable("Frost", 1)
    end
}
